@@ -286,7 +286,9 @@ class Deploy (Resources):
 
     def set_auto_scaling_group_range (self, min_ = None, max_ = None):
         return autoscaling.update_auto_scaling_group (
-            AutoScalingGroupName = self.AUTO_SCALING_GROUP, MinSize = min_, MaxSize = max_
+            AutoScalingGroupName = self.AUTO_SCALING_GROUP,
+            MinSize = min_,
+            MaxSize = max_
         )
 
     def get_hosted_zone_id (self, domain):
@@ -340,7 +342,7 @@ class Deploy (Resources):
 
     def rescale (self, image, current_count, desired_count):
         def runnings ():
-            return [ deployed.id for deployed in self.get_instances_by_image (image) if deployed.state ['Name'] == 'running' ]
+            return [ deployed.id for deployed in self.get_instances_by_image (image, 'running') ]
 
         new_instances = runnings ()
         valids = [ t ['Target']['Id'] for t in self.get_elb_targets () if t ['Target']['Id'] in new_instances and t ['TargetHealth']['State'] != 'draining' ]
@@ -382,7 +384,8 @@ class Deploy (Resources):
             Targets = [{'Id': inst.id, 'Port': port}]
         )
 
-    def remove_image_snapshot (self, image_id):
+    def remove_image (self, image_id):
+        ec2cli.deregister_image(ImageId = image_id)
         owner_id = self.get_images () [0]['OwnerId']
         for s in ec2cli.describe_snapshots (OwnerIds = [owner_id]) ['Snapshots']:
             if s ['Description'].find (image_id) != -1:
@@ -393,22 +396,27 @@ class Deploy (Resources):
         images = self.get_images ()
         print ('unregistering images...')
         for img in images [keep:]:
-            id = img ['ImageId']
-            ec2cli.deregister_image(ImageId = id)
-            self.remove_image_snapshot (id)
+            self.remove_image (img ['ImageId'])
             print ('- image unregistered: {}'.format (id))
 
     def get_elb_targets (self, target_group_arn = None):
         return elb2.describe_target_health (TargetGroupArn = target_group_arn or self.ELB_GROUP_ARN) ['TargetHealthDescriptions']
 
-    def get_instances_by_name (self, name):
-        return list (ec2.instances.filter(Filters=[{'Name': 'tag:Name', 'Values': [name]}]))
+    def filter_state (self, lst, state):
+        if not state:
+            return lst
+        if isinstance (state, str):
+            state = [state]
+        return [inst for inst in lst if inst.state ['Name'] in state]
 
-    def get_instances_by_image (self, image):
-        return list (ec2.instances.filter(Filters=[{'Name': 'image-id', 'Values': [image.id]}]))
+    def get_instances_by_name (self, name, state = None):
+        return self.filter_state (list (ec2.instances.filter(Filters=[{'Name': 'tag:Name', 'Values': [name]}])), state)
+
+    def get_instances_by_image (self, image, state = None):
+        return self.filter_state (list (ec2.instances.filter(Filters=[{'Name': 'image-id', 'Values': [image.id]}])), state)
 
     def get_old_instances_by_image (self, new_image):
-        news = [inst.id for inst in self.get_instances_by_image (new_image) if inst.state ['Name'] == 'running']
+        news = [inst.id for inst in self.get_instances_by_image (new_image, 'running')]
         valids = [ t ['Target']['Id'] for t in self.get_elb_targets () if t ['TargetHealth']['State'] != 'draining' ]
         return [ inst for inst in ec2.instances.filter(Filters=[{'Name': 'tag:Name', 'Values': [self.EC2_NAME]}]) if inst.id not in news and inst.id in valids and inst.state ['Name'] == 'running']
 
